import datetime
import os

from docopt import docopt as docoptinit

haproxy_doc = """
Usage:
    haproxy.py [options]

Options:
    --port=<port>
    --dest=<dest>
    --daemon
    --timeout_server=<t>         Timeout For Server [default: 86400000]
    --timeout_client=<t>         Timeout For Server [default: 86400000]

"""


# haproxy --port port --dest host:port
def haproxy(argv, toy=False):
    docopt = docoptinit(haproxy_doc, argv)
    print(docopt)
    port = int(docopt['--port'])
    opt = {'dest': docopt['--dest'],
           'timeout_server': docopt['--timeout_server'],
           'timeout_client': docopt['--timeout_client']}
    cfg = """
    global
        debug
        log 127.0.0.1 local0
        maxconn 20480

    defaults
        mode tcp
        log     global
        option tcplog
        option tcp-check
        timeout connect 5000
        timeout client  {timeout_client} # 1day 86400 * 1000 ms
        timeout server  {timeout_server}


    frontend shadow1
        bind :9998
        option tcplog
        default_backend haproxy-nodes

    backend haproxy-nodes
        server node-1 {dest} check

    """.format(**opt)
    print(cfg)
    home = os.path.expanduser('~')
    if not os.path.exists('{}/.qbx'.format(home)):
        print('make directory ~/.qbx')
        os.system('mkdir ~/.qbx')
    filename = '{}/.qbx/haproxy-{}.cfg'.format(home, str(int(datetime.datetime.now().timestamp() * 10e6)))
    fp = open(filename, 'w')
    fp.write(cfg)
    fp.close()

    if docopt['--daemon']:
        addon = '--restart always -d --name haproxy-{}'.format(port)
    else:
        addon = '-it --rm'
    cmd = 'sudo docker run {} -v {}:/usr/local/etc/haproxy/haproxy.cfg -p {}:9998 haproxy'
    cmd = cmd.format(addon, filename, port)
    if toy:
        print(cmd)
    else:
        os.system(cmd)


if __name__ == '__main__':
    haproxy(['--port', '123', '--daemon'], toy=True)



import sys


def must_success(func, *args, **kwargs):
    if 'timeout' not in kwargs:
        kwargs['timeout'] = 5
    try:
        r = func(*args, **kwargs)
    except Exception as e:
        print('force exit, statuscode != 200', type(e))
        sys.exit(1)
    if r.status_code >= 300:
        print(r, r.text)
        print('force exit, statuscode != 200')
        sys.exit(1)
    return r



import logging
import os
import sys

help_doc = """
usage: qbx [--help] 
           <command> [<args>]

These are some qbx commands:
    auth_wrapper
    register_kong
    register-kong
    register_ws
    watch_git
    watch_git_http
    pull
    haproxy
    uproxy
"""


def print_help():
    print(help_doc)


def run():
    logging.basicConfig(format='%(asctime)s %(message)s', level=logging.INFO)
    print('argv---', sys.argv)
    if len(sys.argv) == 1:
        print_help()
    else:
        if sys.argv[1] == 'register-kong':
            from .register_kong import register_kong
            register_kong(sys.argv[2:])
        elif sys.argv[1] == 'register_kong':
            logging.warning('use register-kong instead')
            from .register_kong import register_kong
            register_kong(sys.argv[2:])
        elif sys.argv[1] == 'register_ws':
            from .register_ws import register_ws
            register_ws(sys.argv[2:])
        elif sys.argv[1] == 'watch_git':
            from .watchgit import watch_git
            watch_git(sys.argv[2:])
        elif sys.argv[1] == 'watch_git_http':
            from .watchgit import watch_git_http
            watch_git_http(sys.argv[2:])
        elif sys.argv[1] == 'haproxy':
            from .haproxy import haproxy
            haproxy(sys.argv[2:])
        elif sys.argv[1] == 'pull':
            from .pull import pull
            pull(sys.argv[2:])
        elif sys.argv[1] == 'uproxy':
            from .uproxy import uproxy
            uproxy(sys.argv[2:])
        elif sys.argv[1] == 'auth_wrapper':
            from .auth_wrapper import auth_wrapper
            auth_wrapper(sys.argv[2:])
        elif sys.argv[1] == 'kube-start':
            from . import kube_start
            kube_start.start()
        elif sys.argv[1] == 'upload':
            from .upload import upload
            upload(sys.argv[2:])
        else:
            if sys.argv[1] != 'help' and sys.argv[1] != '--help':
                logging.warning('method not regognize')
            print_help()


def qbs():
    print(sys.argv)
    print(os.environ)
    run_cmdline_when_cond(sys.argv[1:])


def run_cmdline_when_cond(cmdline):
    print('cmdline: ', cmdline)
    import subprocess
    import time
    import arrow
    process = subprocess.Popen(cmdline)
    while True:
        if process.poll() is None:
            time.sleep(1)
        else:
            print('program exit itself')
            break
    print('end at', arrow.now(), 'sleep 30s before exit')
    time.sleep(30)


if __name__ == '__main__':
    # from .watchgit import watch_git
    # watch_git(['git+ssh://git@github.com/qbtrade/quantlib.git', 'log_rpc.py'])
    from . import kube_start

    kube_start.start()



#!/usr/bin/env python3

"""
This file needs to be independent, use only basic library

NO SUPPORT for python2

Let Python3.4 can use it


This is entry point for small tools manage the project

Usage:
    qb.py [options] create_docker_image
    qb.py [options] cdi
    qb.py [options] build_curdir [--repo=<repo>]

Options:
    --run                  Run
    --filter=<filter>      Give a filter for parent folder
    --no-cache             Disable cache on docker build
    --always-rebuild       Force re-build the image
    -v                     Verbose Mode, will set_logger(..., debug=True)
    --root=<root>          Root directory [default: .]


create_docker_image:
    for a Dockerfile such as telegraf/0.12-machine/Dockerfile
    it will build an image with tag
    qbtrade/telegraf:0.12-machine
    or with tag
    telegraf:0.12-machine, and put it in private repo

    if config.yaml has
        public: false
    in repo, then push it only to private(aliyun) repo

rsync <folder>:
"""

import hashlib
import json
import logging
import os
import sys
from functools import lru_cache

import arrow
import yaml
from docopt import docopt as docoptinit


@lru_cache()
def get_docker_client():
    print('docker client created')
    import docker
    client = docker.APIClient(version='1.23')
    print(client.version())
    print(client.base_url)
    return client


def get_hash_of_dirs(directory, verbose=0):
    sha1 = hashlib.sha1()
    if not os.path.exists(directory):
        return -1

    try:
        for root, dirs, files in os.walk(directory):
            for names in files:
                if verbose == 1:
                    print('Hashing', names)
                filepath = os.path.join(root, names)
                try:
                    f1 = open(filepath, 'rb')
                    while 1:
                        # Read file in as little chunks
                        buf = f1.read(4096)
                        if not buf:
                            break
                        t = hashlib.sha1(buf).hexdigest()
                        sha1.update(t.encode('utf8'))
                except:
                    continue
                finally:
                    f1.close()
    except:
        import traceback
        # Print the stack traceback
        traceback.print_exc()
        return -2
    return sha1.hexdigest()


def docker_push(tag):
    for line in get_docker_client().push(tag, stream=True):
        line = line.decode('utf8')
        try:
            # print(line, end='')
            js = json.loads(line)
            print(' '.join('{}={}'.format(k, v) for k, v in js.items()))
        except:
            print(line, end='')


def docker_retag(taga, tagb):
    """
    only support
    :param taga: qbtrade/python
    :param tagb: registry.cn-hangzhou.aliyuncs.com/qbtrade/python
    :return:
    """
    if taga == tagb:
        return
    sp = tagb.find(':')
    repo, tag = tagb[:sp], tagb[sp + 1:]
    print('repo', repo, 'tag', tag)
    get_docker_client().tag(taga, repository=repo, tag=tag)


def docker_retag_and_push(taga, tagb):
    docker_retag(taga, tagb)
    docker_push(tagb)


def load_cdi_config(folder):
    # x = os.path.isfile(os.path.join(folder, '.private'))
    if not os.path.isfile(os.path.join(folder, 'config.yaml')):
        return {}
    t = yaml.load(open(os.path.join(folder, 'config.yaml')))
    return t


def docker_retag_and_push_all(tag, param):
    print('push all', tag, param)
    date_str = arrow.now().strftime('%Y%m%d')
    datetime_str = arrow.now().strftime('%Y%m%d-%H%M%S')
    docker_retag_and_push(tag, param)
    docker_retag_and_push(tag, param + '-' + date_str)
    docker_retag_and_push(tag, param + '-' + datetime_str)


def pull_newest_image_before_build(path):
    for line in open(path).read().splitlines():
        if line.strip().startswith('FROM'):
            image = line.split(' ')[1]
            cmd = 'docker pull {}'.format(image)
            print(cmd)
            os.system(cmd)
            break


def handle_folder(folder):
    """
    a folder contains Dockerfile
    build it and upload
    :param folder:
    :return:
    """
    print('handle', folder)
    sp = os.path.split(folder)
    if sp[0] == '.':
        image = sp[1]
        version = 'latest'
    else:
        image = os.path.split(sp[0])[1]
        version = sp[1]

    tag = 'qbtrade/{}:{}'.format(image, version)

    if docopt['--run']:
        parent_hash = get_hash_of_dirs(folder)
        if not docopt['--always-rebuild']:
            if folder in db and db[folder] == parent_hash:
                print('{} hash check sum'.format(folder))
                return
        success = False
        print('------build------', folder)
        pull_newest_image_before_build(os.path.join(folder, 'Dockerfile'))
        for line in get_docker_client().build(path=folder,
                                              dockerfile='Dockerfile',
                                              nocache=docopt['--no-cache'],
                                              tag=tag,
                                              rm=True):
            line = line.decode('utf8')
            try:
                print(json.loads(line)['stream'], end='')
            except:
                print(line, end='')
            if 'Successfully built' in line:
                success = True
        if not success:
            sys.exit(1)
        print('------push------')
        # today_str = arrow.now().strftime('%Y%m%d')
        cfg = load_cdi_config(folder)
        if cfg.get('public', True):
            docker_retag_and_push_all(tag, tag)
        else:
            print('only push to private repo')
        docker_retag_and_push_all(tag, 'registry.cn-hangzhou.aliyuncs.com/' + tag)
        if os.environ['QB_REGION'] == 'alihk':
            docker_retag_and_push_all(tag, 'registry-vpc.cn-hongkong.aliyuncs.com/' + tag)
        else:
            docker_retag_and_push_all(tag, 'registry.cn-hongkong.aliyuncs.com/' + tag)

        db[folder] = parent_hash
    else:
        print('add --run to run')


def build_curdir():
    folder = os.path.abspath(os.path.expanduser(docopt['--root']))
    print('build folder', folder)
    # parent_hash = get_hash_of_dirs(folder)
    # if not docopt['--always-rebuild']:
    #     if folder in db and db[folder] == parent_hash:
    #         print('{} hash check sum'.format(folder))
    #         return
    success = False
    if docopt['--repo']:
        repo = docopt['--repo']
    else:
        repo = os.path.basename(folder)
    tag = 'qbtrade/{}:latest'.format(repo)
    print('------build------', folder, tag)
    for line in get_docker_client().build(path=folder,
                                          dockerfile='Dockerfile',
                                          nocache=docopt['--no-cache'],
                                          tag=tag,
                                          rm=True):
        try:
            line = line.decode('utf8')
            print(json.loads(line)['stream'], end='')
        except:
            print(line, end='')
        if 'Successfully built' in line:
            success = True
    if not success:
        sys.exit(1)
    print('------push------')
    # cfg = load_cdi_config(folder)
    # if cfg.get('public', True):
    #     docker_push(tag)
    #     docker_retag_and_push(tag, tag + '-' + date_str)
    # else:
    #     print('only push to private repo')
    docker_retag_and_push_all(tag, 'registry.cn-hangzhou.aliyuncs.com/' + tag)

    # db[folder] = parent_hash


def create_docker_image():
    """
    This should display failed information if some docker build fails
    :return:
    """
    print('create_docker_image')
    fail = []

    folder = os.path.abspath(os.path.expanduser(docopt['--root']))
    print('build folder', folder)
    for parent, y, files in os.walk(folder):
        if docopt['--filter'] and parent.find(docopt['--filter']) < 0:
            continue
        if 'Dockerfile' in files:
            handle_folder(parent)
    for x in fail:
        print('fail {}'.format(x))


# def show_config():
#     qb.config.log_config()


# def rsync():
#     import getpass
#     cmd = 'rsync -a --delete --exclude=".git/ " {}  alihk9.qbtrade.org:/home/{}/dev'.format(docopt['<folder>'],
#                                                                                             getpass.getuser())
#     print('exec', cmd)
#     os.system(cmd)


# def init_config():
#     # f = os.path.expanduser('~/.qb.jinja2.yaml')
#     # x = open(f).read()
#     # t = jinja2.Template(x)
#     # s = t.render({'QB_REGION': os.getenv('QB_REGION', 'urwork'), 'QB_ENV': os.getenv('QB_ENV', 'debug')})
#     #
#     # open(os.path.expanduser('~/.qbconf'), 'w').write(s)
#     folder = os.path.expanduser('~/.qb')
#     os.makedirs(folder, exist_ok=True)
#
#     open(os.path.join(folder, 'auth_config.yml'), 'w').write("user: tyz\nsecret_path: /qb_rsa\n")


def main():
    # qb.init('qb.py', debug=False, log_config=False)
    logging.basicConfig(level=logging.DEBUG)
    if docopt['cdi'] or docopt['create_docker_image']:
        create_docker_image()
    elif docopt['build_curdir']:
        build_curdir()
    # if docopt['rsync']:
    #     rsync()
    # if docopt['initconfig']:
    #     init_config()
    #     # if docopt['config']:
    #     #     show_config()


if __name__ == '__main__':
    dotfolder = os.path.expanduser('~/.qbtrade/')
    os.makedirs(dotfolder, exist_ok=True)
    from unqlite import UnQLite

    db = UnQLite(os.path.join(dotfolder, 'qbtrade.db'))

    docopt = docoptinit(__doc__)
    print('docopt', docopt)
    main()



import json
import socket

import requests
from docopt import docopt as docoptinit

register_kong_doc = """
Usage:
    register_ws [options]
    
Options:
    --ot             1Token Endpoint (it will only register into  config:v1ws also)
    --v2             Register to v2
    --name=<name>
    --uri=<uri>
    --ip=<ip>
    --port=<port>
"""


def register_ws(argv):
    docopt = docoptinit(register_kong_doc, argv)
    print(docopt)
    dst_list = []
    if docopt['--v2']:
        dst_list.append('wsv2')
    if docopt['--ot']:
        dst_list.append('v1ws')
    for dst in dst_list:
        print('check', dst)
        from . import util
        r = util.must_success(requests.get, f'http://api.qbtrade.org/redis/get?key=config:{dst}&raw=1')
        r = r.json()
        print('get', r)
        port = docopt['--port']
        if not docopt['--ip']:
            ip = socket.gethostbyname(socket.gethostname())
        else:
            ip = docopt['--ip']
        uri = docopt['--uri']
        r[docopt['--name']] = f'ws://{ip}:{port}/{uri}'
        print('new record', r)

        data = {'key': f'config:{dst}', 'value': json.dumps(r)}
        print('going to set', data)
        util.must_success(requests.post, 'http://api.qbtrade.org/redis/set', data)


if __name__ == '__main__':
    # register_kong(['--name', 'pytest', '--ip', '1.2.3.4', '--uris', '/pytest', '--port', '8080'])
    register_ws(['--name', '/pytest', '--uri', '/ws', '--port', '3000', '--ot'])
